
// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

//  Set TEMP_RESOLUTION to the corresponding resolution of used DS18x20 sensor:
//  18S20: 9  (default setting; can be 9,10,11,or 12)
//  18B20: 12
const unsigned short TEMP_RES_B = 12;
const unsigned short TEMP_RES_S = 9;
unsigned short TEMP_RESOLUTION;
unsigned temp;

// family_code
// 0x28: DS18B20
// 0x10: DS18S20
// 0x01: DS1990
// 0xFF: NoID
char family_code;
char family_code_hex[2];

char *text = "000.0000";

char sernum[8];
char sernum_hex[2];
int i;
int colona;


void Resolution()
{
      Ow_Reset(&PORTE, 2);                         // Onewire reset signal
      Ow_Write(&PORTE, 2, 0x33);                   // Issue command Read_ROM
      Delay_us(120);

      family_code = Ow_Read(&PORTE, 2);
      
      switch( family_code )
      {
       case 0x28:
       TEMP_RESOLUTION = TEMP_RES_B;
       break;
       
       case 0x10:
       TEMP_RESOLUTION = TEMP_RES_S;
       break;
       
       case 0x01:
       TEMP_RESOLUTION = 0;
       break;
       
       case 0xFF:
       TEMP_RESOLUTION = 0;
       break;
       
       default:
       TEMP_RESOLUTION = 0;
       break;
      }
      
 // Unique Part Serial Number
     Ow_Reset(&PORTE, 2);                         // Onewire reset signal
     Ow_Write(&PORTE, 2, 0x33);                   // Issue command Read_ROM
     Delay_us(120);

  for(i = 0; i <= 7; i++)
  {
   sernum[i] = Ow_Read(&PORTE, 2);
  }
}

void Read_Temperature()
{
 // Perform temperature reading
    Ow_Reset(&PORTE, 2);                         // Onewire reset signal
    Ow_Write(&PORTE, 2, 0xCC);                   // Issue command SKIP_ROM
    Ow_Write(&PORTE, 2, 0x44);                   // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTE, 2);
    Ow_Write(&PORTE, 2, 0xCC);                   // Issue command SKIP_ROM
    Ow_Write(&PORTE, 2, 0xBE);                   // Issue command READ_SCRATCHPAD

    temp =  Ow_Read(&PORTE, 2);
    temp = (Ow_Read(&PORTE, 2) << 8) + temp;
}

void Display_SerialNumber()
{
 int i;
 colona = 17;

       for(i = 0; i <= 7; i++)
          {
            colona = colona - 2;
            ByteToHex(sernum[i], sernum_hex);
            Lcd_Out(1, colona, sernum_hex);
          }
}

void Display_Type()
{
      switch( family_code )
      {
       case 0x28:
       Lcd_Out(2, 1, "18B20");
       break;

       case 0x10:
       Lcd_Out(2, 1, "18S20");
       break;

       case 0x01:
       Lcd_Out(2, 1, "DS1990");
       break;

       case 0xFF:
       Lcd_Out(2, 1, "NoID  ");
       break;

       default:
       ByteToHex(family_code, family_code_hex);
       Lcd_Out(2, 1, family_code_hex );// ????
       break;
      }
}

void Display_Temperature()
{
     unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
     char temp_whole;
     unsigned int temp2write;
     unsigned int temp_fraction;
     
     temp2write = temp;
     // Check if temperature is negative
     if (temp2write & 0x8000) 
        {
        text[0] = '-';
        temp2write = ~temp2write + 1;
        }

     // Extract temp_whole
     temp_whole = temp2write >> RES_SHIFT ;

     // Convert temp_whole to characters
     if (temp_whole/100)
        text[0] = temp_whole/100  + 48;
     /*else
        text[0] = '0';  */

     text[1] = (temp_whole/10)%10 + 48;             // Extract tens digit
     text[2] =  temp_whole%10     + 48;             // Extract ones digit
     text[3] = '.';
     // Extract temp_fraction and convert it to unsigned int
     temp_fraction  = temp2write << (4-RES_SHIFT);
     temp_fraction &= 0x000F;
     temp_fraction *= 625;

     // Convert temp_fraction to characters
     text[4] =  temp_fraction/1000    + 48;         // Extract thousands digit
     text[5] = (temp_fraction/100)%10 + 48;         // Extract hundreds digit
     text[6] = (temp_fraction/10)%10  + 48;         // Extract tens digit
     text[7] =  temp_fraction%10      + 48;         // Extract ones digit

     // Print temperature on LCD
     Lcd_Out(2, 7, text);
     
     // Print degree character, 'C' for Centigrades
     Lcd_Chr_CP('\'');                             // 176 ASCII for degree symbol
     Lcd_Chr_CP('C');
}

void main()
{
 // ADC_Init();
 CMCON |=7; // Arret de comparateurs
 // ADCON1 ( 0 0 VCFG PCFG )
 // VCFG = 00 ==> Vref+ = AVdd , Vref- = AVss
 // PCFG = 1101 --> AN1 & AN2
 // 00 00 1101 ==> 0x0D
 ADCON1 = 0x0D;

 PORTA = 0 ;
 TRISA = 0x03;
  //ANSELE = 0;                                    // Configure PORTE pins as digital
  TRISE.B2 = 1;                                  // Configure RE2 pin as input

  Lcd_Init();                                    // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);                           // Clear LCD
  Lcd_Cmd(_LCD_CURSOR_OFF);                      // Turn cursor off
  
  // Main loop
  do
  {
    Resolution();
     
    Display_SerialNumber();
    Display_Type();

    switch( family_code )
    {
       case 0x28: // DS18B20
       case 0x10: // DS18S20
       Read_Temperature();
       // Format and display result on Lcd
       Display_Temperature();
       break;

       default:
       break;
    }

    Delay_ms(100);
  } while (1);
}